import { NativeModules } from 'react-native';
const g = global;
const NaClModule = NativeModules.NaclJsi;
if (NaClModule && typeof NaClModule.install === 'function') {
  NaClModule.install();
}
const constants = g.getConstants();
export const ARGON2ID_MEMLIMIT_MIN = constants.ARGON2ID_MEMLIMIT_MIN;
export const ARGON2ID_MEMLIMIT_MAX = constants.ARGON2ID_MEMLIMIT_MAX;
export const ARGON2ID_MEMLIMIT_INTERACTIVE = constants.ARGON2ID_MEMLIMIT_INTERACTIVE;
export const ARGON2ID_MEMLIMIT_MODERATE = constants.ARGON2ID_MEMLIMIT_MODERATE;
export const ARGON2ID_MEMLIMIT_SENSITIVE = constants.ARGON2ID_MEMLIMIT_SENSITIVE;
export const ARGON2ID_OPSLIMIT_MIN = constants.ARGON2ID_OPSLIMIT_MIN;
export const ARGON2ID_OPSLIMIT_MAX = constants.ARGON2ID_OPSLIMIT_MAX;
export const ARGON2ID_OPSLIMIT_INTERACTIVE = constants.ARGON2ID_OPSLIMIT_INTERACTIVE;
export const ARGON2ID_OPSLIMIT_MODERATE = constants.ARGON2ID_OPSLIMIT_MODERATE;
export const ARGON2ID_OPSLIMIT_SENSITIVE = constants.ARGON2ID_OPSLIMIT_SENSITIVE;
export const ARGON2ID_SALTBYTES = constants.ARGON2ID_SALTBYTES;
export const BOX_PUBLIC_KEY_LENGTH = constants.BOX_PUBLIC_KEY_LENGTH;
export const BOX_SECRET_KEY_LENGTH = constants.BOX_SECRET_KEY_LENGTH;
export const BOX_NONCE_LENGTH = constants.BOX_NONCE_LENGTH;
export const BOX_SEED_LENGTH = constants.BOX_SEED_LENGTH;
export const SECRETBOX_KEY_LENGTH = constants.SECRETBOX_KEY_LENGTH;
export const SECRETBOX_NONCE_LENGTH = constants.SECRETBOX_NONCE_LENGTH;

/**
 * Encodes a buffer into a base64 string
 *
 * @throws when the buffer is not an `Uint8Array`
 */
export function encodeBase64(buffer) {
  return g.encodeBase64(buffer.buffer);
}

/**
 * Decodes a base64 string into a buffer
 *
 * @throws when the given string is not a valid base64 string
 */
export function decodeBase64(toEncode) {
  return new Uint8Array(g.decodeBase64(toEncode));
}

/**
 * Encodes a buffer into an hexadecimal string
 *
 * @throws when buffer is not an `Uint8Array`
 */
export function encodeHexadecimal(buffer) {
  return g.encodeHexadecimal(buffer.buffer);
}

/**
 * Decodes an hexadecimal string into an Uint8Array
 *
 * @throws when the given string is not a valid hexadecimal string
 */
export function decodeHexadecimal(toDecode) {
  return new Uint8Array(g.decodeHexadecimal(toDecode));
}

/**
 * Encodes a buffer into a UTF-8 string
 *
 * @throws when buffer is not an `Uint8Array`
 */
export function encodeUtf8(buffer) {
  return g.encodeUtf8(buffer.buffer);
}

/**
 * Decodes a UTF-8 string into an `Uint8Array`
 */
export function decodeUtf8(input) {
  return new TextEncoder().encode(input);
}

/**
 * Generates a key to be used with aesEncrypt and aesDecrypt
 */
export function aesGenerateKey() {
  const key = g.aesGenerateKey();
  return new Uint8Array(key);
}

/**
 * Encrypts a message with AES-256-GCM using the given key
 *
 * @throws when message is not an `Uint8Array`
 * @throws when key is not an `Uint8Array`
 * @throws when key length is incorrect
 */
export async function aesEncrypt(message, key) {
  const aesResult = g.aesEncrypt(message.buffer, key.buffer);
  return {
    encrypted: new Uint8Array(aesResult.encrypted),
    iv: new Uint8Array(aesResult.iv)
  };
}

/**
 * Decrypts a cipher text with AES-256-GCM using the given key and the initialization vector
 *
 * @throws when cipherText is not an `Uint8Array`
 * @throws when key is not an `Uint8Array`
 * @throws when key length is incorrect
 * @throws when iv is not an `Uint8Array`
 * @throws when iv length is not an `Uint8Array`
 * @throws when the decryption failed
 */
export async function aesDecrypt(cipherText, key, iv) {
  const decrypted = g.aesDecrypt(cipherText.buffer, key.buffer, iv.buffer);
  return new Uint8Array(decrypted);
}

/**
 * Generates a key pair to be used with boxSeal and boxOpen
 */
export function boxGenerateKey() {
  const keyPair = g.boxGenerateKey();
  return {
    publicKey: new Uint8Array(keyPair.publicKey),
    secretKey: new Uint8Array(keyPair.secretKey)
  };
}

/**
 * Encrypts a message using the recipient's public key and the sender's secret key
 *
 * @throws when message is not an `Uint8Array`
 * @throws when recipientPublicKey is not an `Uint8Array`
 * @throws when recipientPublicKey length is incorrect
 * @throws when senderSecretKey is not an `Uint8Array`
 * @throws when senderSecretKey length is incorrect
 * @throws when the encryption didn't succeed successfully
 */
export function boxSeal(message, recipientPublicKey, senderSecretKey) {
  const encrypted = g.boxSeal(message.buffer, recipientPublicKey.buffer, senderSecretKey.buffer);
  return new Uint8Array(encrypted);
}

/**
 * Verifies and decrypts a ciphertext produced by `boxSeal`
 *
 * @throws when cipherText is not an `Uint8Array`
 * @throws when senderPublicKey is not an `Uint8Array`
 * @throws when senderPublicKey length is incorrect
 * @throws when recipientSecretKey is not an `Uint8Array`
 * @throws when recipientSecretKey length is incorrect
 * @throws when the verification failed
 */
export function boxOpen(cipherText, senderPublicKey, recipientSecretKey) {
  const decrypted = g.boxOpen(cipherText.buffer, senderPublicKey.buffer, recipientSecretKey.buffer);
  return new Uint8Array(decrypted);
}

/**
 * Generates a key to be used with secretboxSeal and secretboxOpen
 */
export function secretboxGenerateKey() {
  const secretKey = g.secretboxGenerateKey();
  return new Uint8Array(secretKey);
}

/**
 * Encrypts a message with the given secret key
 *
 * @throws when message is not a `Uint8Array`
 * @throws when secretKey is not a `Uint8Array`
 * @throws when secretKey length is incorrect
 * @throws when the encryption didn't complete successfully
 */
export function secretboxSeal(message, secretKey) {
  const encrypted = g.secretboxSeal(message.buffer, secretKey.buffer);
  return new Uint8Array(encrypted);
}

/**
 * Verifies and decrypts a ciphertext produced by `secretboxSeal`
 *
 * @throws when cipherText is not an `Uint8Array`
 * @throws when secretKey is not an `Uint8Array`
 * @throws when secretKey length is incorrect
 * @throws when the verification failed
 */
export function secretboxOpen(cipherText, secretKey) {
  const decrypted = g.secretboxOpen(cipherText.buffer, secretKey.buffer);
  return new Uint8Array(decrypted);
}

/**
 * Generates a key pair for signing and verifying signatures
 */
export function signGenerateKey() {
  const keyPair = g.signGenerateKey();
  return {
    publicKey: new Uint8Array(keyPair.publicKey),
    secretKey: new Uint8Array(keyPair.secretKey)
  };
}

/**
 * Signs a message with the given secret key
 *
 * @throws when message is not a `Uint8Array`
 * @throws when secretKey is not a `Uint8Array`
 * @throws when secretKey length is incorrect
 */
export function signDetached(message, secretKey) {
  const signature = g.signDetached(message.buffer, secretKey.buffer);
  return new Uint8Array(signature);
}

/**
 * Verifies that the signature is valid given the message the public key
 *
 * @throws when message is not a `Uint8Array`
 * @throws when publicKey is not a `Uint8Array`
 * @throws when publicKey length is incorrect
 * @throws when signature is not a `Uint8Array`
 */
export function signVerifyDetached(message, publicKey, signature) {
  return g.signVerifyDetached(message.buffer, publicKey.buffer, signature.buffer);
}

/**
 * Hash a password using the Argon2id algorithm
 *
 * @throws when password is not a `Uint8Array`
 * @throws when the hash computation didn't complete successfully
 */
export async function argon2idHash(password, iterations, memoryLimit) {
  return g.argon2idHash(password.buffer, iterations, memoryLimit);
}

/**
 * Verifies that `hash` is a valid `password` verification string
 *
 * @throws when password is not a `Uint8Array`
 */
export async function argon2idVerify(hash, password) {
  return g.argon2idVerify(hash, password.buffer);
}

/**
 * Derives a key of the given length
 *
 * @throws when password is not a `Uint8Array`
 * @throws when salt is not a `Uint8Array` or its size is not `ARGON2ID_SALTBYTES`
 */
export async function argon2idDeriveKey(password, salt, keyLength, iterations, memoryLimit) {
  const key = g.argon2idDeriveKey(password.buffer, salt.buffer, keyLength, iterations, memoryLimit);
  return new Uint8Array(key);
}

/**
 * Returns a buffer of the given size filled with random bytes
 *
 * @throws When the given size is incorrect
 */
export function getRandomBytes(size) {
  const randomBytes = g.getRandomBytes(size);
  return new Uint8Array(randomBytes);
}
//# sourceMappingURL=nacl.js.map